#!/usr/bin/perl -w

# sudoku3.pl -- a simple program to generate sudoku patterns. The 
#  solveBoard() function can also solve sudoku games.
#
#  Copyright (C) 2008 Dave Jacobowitz
#

use Data::Dumper;

use strict;
use warnings qw(all);


my $debug = 0;

my $m1  = 0x55555555;
my $m2  = 0x33333333;
my $m4  = 0x0f0f0f0f;


sub dprint {

};

# these clever popcount algos are from wikipedia. Too bad I'm not in
# C, I'm sure this processsor has a popcount insn. (return number of binary 
# 1's # in an integer)
sub popcount {
 my $x = shift;
 $x -= ($x >> 1) & $m1; 
 $x = ($x & $m2) + (($x >> 2) & $m2); 
 $x = ($x + ($x >> 4)) & $m4; 
 $x += $x >>  8;  
 $x += $x >> 16; 
 # $x += $x >> 32;  
 return $x & 0x3f;
}

# return the bit location of the n'th 1 in an integer
sub findNth1 {
 my $x = shift;
 my $n = shift;
 my $p = 0;
 my $c = 0;
 while ($x) {
  if ($x & 0x1) {
   if ($c == $n) { return $p; };
   $c++;
  }
  $x = $x >> 1;
  $p++;
 }
 return -1;
}


sub coordToIndex {
 my $i = shift;
 my $j = shift;
 my $e = shift;
 my $idx = $j*$e + $i;
 return $idx;
}

# create a completely empty sudoku board. This is the starting point
# for creating a sudoku game.
sub makeEmpty {
 my $e = shift;
 my $b = {};

 my $e2 = $e * $e;
 for (my $i=0;$i<$e2;$i++) {
  $b->{$i} = { v => 'u',
               s => 1 };
 }
 return $b;
}



# create a sudoku game. Note that making a game is basically the same
# as solving an empty board; the solution is not unique, that's why
# we can get different games out of this algo.
sub makeBoard3 {
 my $edge = shift;
 my $e2   = $edge*$edge;
 my $b = makeEmpty($edge);
 calcCandidates($b,$edge);
 my $r = solveBoard($b,$edge);
 return $r;
}

# this calculates the candidate values for each unfilled location on the 
# board. Called on a blank board, for example, then all values are possible,
# so every location has a full mask (b111111111 for a 9x9 board) set. But
# this routine should also work on a partially completed board.
sub calcCandidates {
 my $b = shift;
 my $e = shift;
 
 my $e2 = $e * $e;
 my $se = sqrt($e);

 for (my $idx=0;$idx<$e2;$idx++) {
  if ($b->{$idx}{v} eq 'u') {
   my $col = $idx % $e;
   my $row = int($idx / $e);
   my $box = $se * int($row / $se) + int($col / $se);
   my $box_corner_row = $se * int($box / $se);
   my $box_corner_col = $se * ($box % $se);
   my $candidates =  (2 ** $e) -1;
   for (my $i=0;$i<$e;$i++) {
    my $row_idx = $i + $row * $e;
    my $col_idx = $col + $i * $e;
    my $box_col = $box_corner_col + ($i % $se);
    my $box_row = $box_corner_row + int($i / $se);
    my $box_idx = $box_col + $box_row * $e;

    my $v1 = $b->{$row_idx}{v};
    my $v2 = $b->{$col_idx}{v};
    my $v3 = $b->{$box_idx}{v};
    foreach my $v ($v1, $v2, $v3) {
     if (($v ne 'u') && ($v ne ' ')) {
      my $mask = ~(0x1 << $v);
      $candidates &= $mask;
     }
    }
   }
   $b->{$idx}{c} = $candidates;
   $b->{$idx}{pc} = popcount($candidates);
  }
 } 
}



# this routine sets a location in the board to a specific value. It 
# also updates the "possibles" for all it's row, column, and box neighbors.
sub setItem {
 my $b   = shift;
 my $e   = shift;
 my $idx = shift;
 my $v   = shift;

 my $se = sqrt($e);

 my $col = $idx % $e;
 my $row = int($idx / $e);
 my $box = $se * int($row / $se) + int($col / $se);
 my $box_corner_row = $se * int($box / $se);
 my $box_corner_col = $se * ($box % $se);
 
 my $andmask = ~(0x1 << $v);

 for (my $i=0;$i<$e;$i++) {
  my $row_idx = $i + $row * $e;
  my $col_idx = $col + $i * $e;
  my $box_col = $box_corner_col + ($i % $se);
  my $box_row = $box_corner_row + int($i / $se);
  my $box_idx = $box_col + $box_row * $e;

  foreach my $_idx ($row_idx, $col_idx, $box_idx) {
   $b->{$_idx}{c} &= $andmask;
   $b->{$_idx}{pc} = popcount($b->{$_idx}{c});
  }
 }
 $b->{$idx}{v} = $v;
}



# you give me a sudoku board, I solve it. If you give me an empty board
# or one that is partly empty and has multiple solutions, then you're going
# to get one random solution. Essentially, this routine can create new 
# boards as well as solve existing ones; it's the same process.
sub solveBoard {
 # the recursion gets deep on a 16x16 board. I probably should
 # de-recursivy this algorithm, but it's nice the way it is, 
 # even if a little slow.
 no warnings 'recursion';   

 my $bc = shift;
 my $e  = shift;

 my $e2 = $e*$e;
 my $se = sqrt($e);


 # first get only unset cells;
 my $ubc = {};
 foreach my $idx (keys %$bc) {
  if ($bc->{$idx}{v} eq 'u') {
   $ubc->{$idx} = $bc->{$idx};
  }
 } 

 # now sort on the order of constrainment
 my @cidxs = sort { $ubc->{$a}{pc} <=>
                    $ubc->{$b}{pc}     } keys %$ubc; 
 my $idx_count = scalar @cidxs;

 if ($debug) {
  print makeBar($e2- $idx_count,$e2-1) . "\n";
 };

 my $idx_to_set = $cidxs[0];
 my $candidates = $bc->{$idx_to_set}{c};
 my $pc         = $bc->{$idx_to_set}{pc};
 
 while ($pc) {
  my $choice = findNth1($candidates,int(rand($pc)));
  my %bhc = %$bc; my %bhcc = %bhc; my $bcc = \%bhcc;
  setItem($bcc,$e,$idx_to_set,$choice);
  if ($idx_count == 1) {
   return { b => $bcc, s => 1 };
  } else {
   my $r = solveBoard($bcc,$e);
   if ($r->{s}) {
    return { b => $bcc, s => 1 };
   } else {
    $candidates &= ~(0x1 << $choice);
    $pc = popcount($candidates);
   }
  }
 }
 return { s => 0 };
}

sub shuffle {
 my $a = shift;
 my $n = scalar @{$a};
 while ($n) {
  my $k = int(rand($n));
  $n--;
  my $t = $a->[$n];
  $a->[$n] = $a->[$k];
  $a->[$k] = $t;
 }
 return $a;
}


# pretty(ish) print our completed board
sub printBoard2 {
 my $b = shift;
 my $e = shift;

 my $se = sqrt($e);
 my $os = '';
 my $l = $e*$e;

 $os .= "+" . join("",map("--",(0 .. $e+$se-1))) . "\n";
 $os .= "| ";
 for (my $i=0;$i<$l;$i++) {
  my $v = $b->{$i}{v};
  
  $os .= ($b->{$i}{s}) ? 
         (($v eq 'u') ? 'u ' :
	  sprintf("%1.1x ",$v)) : "  ";

  if (($i % $se) == ($se - 1)) { 
   $os .= "| "; 
  }
  if (($i % $e) == ($e - 1)) {
   if ((int($i / $e) % $se) == ($se - 1)) {
    $os .= "\n|";
    $os .= join("",map("--",(0 .. $e+$se-1))) . "\n";
    $os .= "| ";
   } else {
    $os .= "\n| ";
   }
  };
 }
 chop($os); chop($os); chop($os);
 return $os;
}


# take a board and set some elements to be invisible
sub hideElems {
 my $b = shift;
 my $n = shift;
 my $l = scalar keys %$b;
 my @ll = (0 .. ($l-1));
 my $hl = shuffle(\@ll);

 for (my $i=0;$i<$n;$i++) {
  $b->{$hl->[$i]}{s} = 0;
 }
}


sub makeBar {
 my $c = shift;
 my $m = shift;
 my $q = shift;
 if (!defined($q)) {  $q = '='; };
 my $f = $c/$m;
 my $l = int(($f * 75) + 0.5);
 my $s = join($q,map('',(0 .. $l)));
 return $s;
};

#######
# MAIN
#######


my $games  = 1;
my $edge   = 9;
my $blanks = int($edge*$edge/3);


while (@ARGV) {
 my $arg = shift @ARGV;
 if ($arg =~ /^-g/) {
  $games = shift @ARGV;
 } elsif ($arg =~ /^-h/) {
  $edge = 16;
  $blanks = int($edge*$edge/3);
 } elsif ($arg =~ /^-b/) {
  $blanks = shift @ARGV;
 } elsif ($arg =~ /^-d/) {
  $debug = 1;
 }
}



dprint "\n***\n*** Dave's Sudoku Generator\n***\n\n" .
      " [ -b blank_count ]  # number of cells to hide\n" .
      " [ -h ]              # do a hex game (0-f) rather than 1-9\n" .
      " [ -d ]              # turn on debug\n" .
      " [ -g game_count ]   # number of games to generate\n\n";


for (my $count = 0;$count<$games;$count++) {
 my $ok = 0;
 my $r = undef;
 while(!$ok) {
  $r  = makeBoard3($edge);
  $ok = $r->{s};
 }
 hideElems($r->{b},$blanks);
 print "Game $count\n";
 print printBoard2($r->{b},$edge);
 print "\n";
}

print "\n";


